"""
    _load_json(file::String = "spectral-indices-dict.json")

Load a specified JSON file from the data folder.

# Arguments

  - `file::String = "spectral-indices-dict.json"`: The name of the JSON file to be loaded.

# Returns

  - `object`: The parsed JSON content from the specified file.

# Examples

```julia
# Load the default JSON file
data = _load_json()

# Load a specific JSON file
data = _load_json("my-custom-indices.json")
```
"""
function _load_json(filename::String="spectral-indices-dict.json")
    fileloc = joinpath(dirname(@__FILE__), "..", "data", filename)
    if isfile(fileloc)
        return JSON.parsefile(fileloc)
    else
        error("The JSON file is not in the SpectralIndices/data folder")
    end
end

"""
    _get_indices(online::Bool = false)

Retrieve the JSON data of spectral indices.

# Arguments

  - `online::Bool = false`: Whether to retrieve the most recent list of indices directly from the GitHub repository (online) or from the local copy (offline).

# Returns

  - `dict`: A dictionary containing the spectral indices data.

# Examples

```julia
# Retrieve the spectral indices from the local copy (offline)
indices = _get_indices()

# Retrieve the most recent spectral indices from the online repository
indices = _get_indices(true)
'''
```
"""
function _get_indices(
    online::Bool=false;
    filename="spectral-indices-dict.json",
    fileloc=joinpath(dirname(@__FILE__), "..", "data", filename),
)
    if online
        indices_loc = Downloads.download(
            "https://raw.githubusercontent.com/awesome-spectral-indices/awesome-spectral-indices/main/output/spectral-indices-dict.json",
            fileloc,
        )
        indices = JSON.parsefile(indices_loc)
    else
        indices = _load_json()
    end

    return indices["SpectralIndices"]
end

function _create_indexfun(
    index_dict::Dict{String,Any}=_get_indices();
    filename::String="indices_funcs.jl",
    fileloc=joinpath(dirname(@__FILE__), filename),
)
    open(fileloc, "w") do file
        for (index_name, index_info) in index_dict
            short_name = index_info["short_name"]
            formula = index_info["formula"]
            formula = replace(formula, "**" => "^")
            bands = index_info["bands"]
            bands_args = join(bands, "::Number, ") * "::Number"

            write(file, "\n$(short_name)_func($(bands_args)) = $formula\n")
        end

        for (index_name, index_info) in index_dict
            short_name = index_info["short_name"]
            write(file, "indices_funcs[\"$index_name\"] = $(short_name)_func\n")
        end
    end
end
